/*
   ----------------            creepy figures.pov               ------------------------
   ----------------             3. february 2018                ------------------------
   ----------------       copyright by Norbert W. Kern          ------------------------
   ----------------      norbert-werner.kern@t-online.de        ------------------------
*/

// see http://news.povray.org/povray.binaries.images/thread/%3Cweb.5a7f6d04f7f763caf8cfcbc0%40news.povray.org%3E/

#version 3.7;

#declare RAD = 2;               // final render was 16 - very slow
                
global_settings {
        #declare a_g = 1;
        assumed_gamma a_g
        max_trace_level 4
        noise_generator 2 
        #if(RAD > 0)
                radiosity {
                        pretrace_start 0.08
                        pretrace_end   0.04/RAD
                        count 30*RAD                
                        nearest_count RAD      
                        error_bound 2/RAD
                        low_error_factor 0.5      
                        recursion_limit 1      
                        gray_threshold 0  
                        minimum_reuse 0.015 
                        brightness 1      
                        adc_bailout 0.005
                        normal on
                        media off
                }
        #end
}        

#include "stdinc.inc" 

#declare camloc = <1,1.5,-5>; 
#declare lookat = <0,2.25,0>;
#declare camang = 86.7;
#declare camrig = x*image_width/image_height;

camera {
        location camloc 
        look_at lookat
        right camrig 
        up y 
        angle camang
}

background {color rgb <0.52,0.5,0.48>*1.4} 

fog {
        color rgb <0.52,0.5,0.48>
        fog_type 2
        distance 100
        fog_alt 20
        turbulence 0.15
        lambda 2.3
}    

//Sun

#declare LPos = <-50,13,-100*0.25>*100;

light_source {
        LPos, 
        srgb <2.42,2.23,1.87>*0.25*1.4
        media_attenuation 0
        media_interaction 1
        parallel point_at 0
}

//Macros

#macro hsl (H, S, L)

#if (H < 1/3) #local R = min (2-H*6,1); #end
#if (H > 2/3) #local R = min ((H-2/3)*6,1); #end
#if (H >= 1/3 & H<=2/3) #local R = 0; #end

#if (H <= 1/3) #local G = min (H*6,1); #end
#if (H > 1/3 & H < 2/3) #local G = min (2-(H-1/3)*6,1); #end
#if (H >= 2/3) #local G = 0; #end

#if (H > 1/3&H<=2/3) #local B = min ((H-1/3)*6,1); #end
#if (H > 2/3) #local B = min (2-(H-2/3)*6,1); #end
#if (H <= 1/3) #local B = 0; #end

#local RGB = ((1-S)+S*<R,G,B>)*L;
srgb <RGB.x, RGB.y, RGB.z> 

#end

#macro rand_ext (M, D, Seed)

(M+(rand (Seed)-0.5)*2*D)

#end

#macro v_rand_ext (M, D, Seed)

#local MV = M+<0,0,0>;
#local DV = D+<0,0,0>;
(<rand_ext (MV.x, DV.x, Seed), rand_ext (MV.y, DV.y, Seed), rand_ext (MV.z, DV.z, Seed)>)

#end

#macro v_debug (V)

concat ("<",str (V.x,0,3),",",str (V.y,0,3),",",str (V.z,0,3),">") 

#end

//Ground

#declare Rade = 30000;  //Earth radius
#declare S = function {sqrt (pow (x, 2)+pow (y, 2)+pow (z, 2))-Rade-1.7}

#declare N = function {f_ridged_mf (x, y, z, 0.8, 1.5, 3, 0.8, 0.2, 2)}

#declare N2 = 
function {
        pigment {
                crackle 
                color_map {
                        [0.1    rgb 0]
                        [1      rgb 1]
                } 
                scale 0.04 
                offset 0.5
        }
}

#declare GroundTex =
texture {
        pigment {
                crackle 
                color_map {
                        [0      hsl (0.13-0.06, 0.3, 0.5)]
                        [0.7    hsl (0.19-0.06, 0.3, 0.75)]
                } 
                solid 
                offset 0.5 
                scale 0.04
        }
        finish {ambient 0 diffuse 0.7*0.7 brilliance 1.2}
}

#declare Ground =
isosurface {
        function {
                S (x,y,z)
                +f_noise3d (x*0.05,0,z*0.1-7.8)*4
                -N (x*0.75,y*0.75,z*0.75)*0.17
                -N2 (x,y,z).x*0.15
        }
        contained_by {sphere {0, Rade+10}}
        translate -y*Rade
}

object {Ground texture {GroundTex}}

//Throw in some stones

#declare Seed = seed (44224);
#declare StoneTex =
texture {
        normal {granite 0.5 scale 0.1}
        finish {diffuse 0.7*0.7 brilliance 1.3}
}

#declare SphereFunc = function {f_sphere (x,y,z,1)}

#macro PlaceStones (Radius, Number, Size, Rand, Seed)

#local C = 0;
#while (C < Number)
        #undef N
        #local PosXZ = vrotate (z*rand_ext (0, Radius,Seed),y*rand (Seed)*360);
        #local N = <0,0,0>;
        #local Pos=trace (Ground, PosXZ-100*y, y, N);
        #if (N.x != 0 & N.y != 0 & N.z != 0)
                #local RandTrans = v_rand_ext (0, 20, Seed);
                #local RandSize = Size*v_rand_ext (1, Rand, Seed);
                #local RandScale = v_rand_ext (Size*11, Size*2, Seed);
                #undef ModFunc
                #local ModFunc =
                function {
                        pigment {
                                crackle
                                color_map {
                                        [0      srgb 0]
                                        [0.3    srgb 0.5]
                                        [1      srgb 1]
                                }
                                turbulence 0.1
                                scale RandScale
                                translate RandTrans
                        }
                }
                isosurface {
                        function {
                                SphereFunc (x,y,z)
                                +ModFunc (x,y,z).x*0.3
                        }
                        contained_by {sphere {0, 1}}
                        scale RandSize
                        translate Pos
                        texture {
                                StoneTex
                                pigment {
                                        granite
                                        color_map {
                                                [0      hsl (rand_ext (0.15-0.06, 0.1, Seed), 0.75*rand_ext (0.2 ,0.1, Seed), rand_ext (0.4, 0.2, Seed))]
                                                [1      hsl (rand_ext (0.13-0.06, 0.2, Seed), 0.75*rand_ext (0.2, 0.1, Seed), rand_ext (0.7, 0.3, Seed))]
                                        }
                                        scale 0.1
                                }
                        }
                }
        #else
                #warning concat ("No intersection at PosXZ: ",v_debug (PosXZ),"\n")
        #end
        #local C = C+1;
#end
  
#end

PlaceStones (300, 2000, 0.17, 0.7, Seed)
PlaceStones (100, 5000, 0.07, 0.5, Seed) 

//________________________________________________________________________________________________________________________________________________________________

global_settings {
        irid_wavelength srgb <254,182,89>/255
}

#local T_2317 =
material {
	texture {
		pigment {color rgb <254,182,89>/255}
	        normal {
                	average
                	normal_map {
        		        [1      bumps 
                                        scale 0.0175 
                                        bump_size 0.2
                                ]
                		[1      bumps 
                                        scale 0.35 
                                        bump_size 0.2
                                ]
                		[1      function {(f_ridged_mf ((4+x)/0.3, y/0.3, z/0.3, 0.5, 2.7, 4, 1, 1.5, 0)-1.8)*0.5}
                                	scale 0.35
                                	bump_size 0.3
                                	slope_map {
                                		[0      <0,1>]
                                		[0.25   <1,0>]
                                		[1      <1,0>]
                                	}
                                ]
                	}
                        accuracy 0.003
                }
		finish {
                	ambient 0
                	diffuse 0.16*2
                	specular 0.8
                	roughness 0.11
                	metallic
                	reflection {0.2, 0.4 metallic}
                        irid {0.07 thickness 0.5 turbulence 0.2}
                }
	}
	interior {ior 2}
	scale 4
}

#declare C = 1;
#declare clock_ = 0;   
 
#declare F_ = 
function {
        pigment {
                crackle 
                metric 0.05 
                form x/1e9 
                poly_wave 0.4 
                color_map {
                        [0      rgb 1]
                        [1      rgb 0]
                }
        }
}

#declare T2223 =
material {
        texture {
                function {F_ (x,y,z).x}
                triangle_wave 
                frequency 2 
        	texture_map {
        	        [0.7    pigment {
        	                	aoi
        	                	color_map {
        	                		[0      srgb x]
        	                		[0.5    srgb <0.1,0.45,0.8>]
        	                		[1      srgb x]
        	                	}
        	                }
                                normal {
                                        function {F_ (x,y,z).x}
                                        triangle_wave 
                                        frequency 2
                                        bump_size -1
                                }
        	                finish {
        	                	specular albedo 0.25
        	                	metallic
        	                	roughness 0.025
        	                	diffuse 0.6
        	                	brilliance 2
        	                	reflection {0.125 metallic}
        	                	conserve_energy
        	                	irid {0.8 thickness 0.1 turbulence 0.5}
        	                }
        	        ]
        	        [1      pigment {color srgb <0.7,0.7,0.7>}
        		        finish {
        		        	diffuse 0.2
        		        	brilliance 4
        		        	reflection {0.125, 0.25 fresnel on}
        		        	conserve_energy
        		        	phong 2
        		        	phong_size 200
        		        	specular 0.3
        		        	roughness 0.05
        		        }
        		        normal {granite 0.07 scale 0.0007 accuracy 0.1} 
                        ]
        	}
        	scale 0.2
        }
        interior {ior 2}
}

//==================================================
//Bounding Box....... from x,y,z=(-0.31928,0.00135714,-0.18983)
//                      to x,y,z=(0.101185,0.832934,0.110534)
//                 size dx,dy,dz=(0.420465,0.83157686,0.300364)
//                  center x,y,z=(-0.1090475,0.41714557,-0.039648)
//                       diagonal 0.979045158717645
//Surface area................... 0.267393497859115
//             Smallest face area 2.87147660446792E-10
//              Largest face area 9.64448967020712E-5
//Memory allocated for geometry: 2 MBytes
// 
//==================================================

#include "figure_POV_geom.inc"

#declare set_up5 = 
object {
        figure_
        translate <0,-0.00135714,0>
        scale 65/0.83157686
        rotate y*-30
} 

#macro SaveVectors (vectors, filename)

#declare Save_precision = 6;
#declare Save_columns = 5;

#local N_vec = dimension_size (vectors, 1);
#fopen fd filename write
#write (fd, concat ("\narray [", str (N_vec,0,0), "]\n{"))

#local i = 0;
#while (i < N_vec)
        #write (fd, concat ("<", str (vectors [i].x,0,Save_precision), ", ", str (vectors [i].y,0,Save_precision), ", ", str (vectors [i].z,0,Save_precision), ">"))
        #local i = i+1;
        #if (i < N_vec)
                #write (fd, ", ")
        #end
        #if (mod (i, Save_columns) = 0)
                #write (fd, "\n")
        #end
#end

#write (fd, concat ("}\n"))
#fclose fd 
    
#end

#macro RandOnSurface (Obj, rsd, thresh)

#local mn = min_extent (Obj);
#local mx = max_extent (Obj);
#local dist = 1000000;

#while (dist > thresh)
        #local flag = 0;
        #while (flag = 0)
                #local rfrom = VRand_In_Box (mn, mx, rsd);
                #local rdir = vrotate (<1,0,0>, <360*rand (rsd),360*rand (rsd),360*rand (rsd)>);
                #declare Norm = <0,0,0>;
                #declare Norm2 = <0,0,0>;
                #declare hit = trace (Obj, rfrom, rdir, Norm);                                  // the interesting part
                #if (vlength (Norm) != 0)
                        #local flag = 1;
                        #declare hit2 = trace (Obj, hit, -Norm, Norm2);     
                #end
        #end
        #local dist = vlength (hit-rfrom);
#end 
#if (vlength (Norm2) != 0)
        #declare hit_ = (hit+hit2)*0.5;
#else
        #declare hit_ = hit);
#end 
#local result = hit_;
(result)

#end 

#declare rsd = seed (67);
#declare p_count = 10000;

#declare p_a = array [p_count];
#declare n_a = array [p_count]; 
#declare h_a = array [p_count]; 

#declare F1 = concat ("point_Figure.inc"); 
#declare F2 = concat ("norm_Figure.inc"); 
#declare F3 = concat ("hit_Figure.inc"); 
        
#if (!file_exists (F1))
        #local c = 0;
        #while (c < p_count)
                #declare p_a [c] = RandOnSurface (set_up5, rsd, 0.05*14);
                #declare n_a [c] = Norm;
                #declare h_a [c] = hit;
                #if (vlength (Norm2) = 0 | inside (set_up5, p_a [c]) < 0.5)
                        #declare c = c-1;
                #end
                #local c = c+1;
                #if (mod (c, 20) = 0) 
                        #debug concat ("found: ", str (c,0,0), "\n") 
                #end 
        #end
        SaveVectors (p_a, F1)
        SaveVectors (n_a, F2)
        SaveVectors (h_a, F3)                                                                   // end interesting part
#end

#declare p_a = #include F1;
#declare n_a = #include F2;
#declare h_a = #include F3;

#declare set_up5_1 =            // creepy figure
union {
        #local c = 0;
        #while (c < p_count)
                box {
                        -0.05*4, 0.05*4
                        Point_At_Trans (n_a [c])            
                        translate p_a [c]
                }
                #local c = c+1;
        #end
}

object {
        set_up5_1 
        scale 0.065 
        rotate y*-180
        translate <-2.1,0,0>
        material {T_2317}              
}

#declare set_up5_2 =            // hits on original mesh - for comparison
union {
        #local c = 0;
        #while (c < p_count)
                box {
                        -0.01*4, 0.01*4
                        Point_At_Trans (n_a [c])
                        translate h_a [c]
                }
                #local c = c+1;
        #end
}

object {
        set_up5_2 
        scale 0.065 
        rotate y*-180
        translate <-2.1,0,0>
        material {T2223}       
}

#declare set_up5_3 =            // spikes a la Gormley - http://meganrodgersart.blogspot.de/2013/05/antony-gormley-wire-sculptures.html
union {
        #local c = 0;
        #while (c < p_count)
                union {
                        cone {
                                <0,0,0>, 0.05, <0,14,0>*(0.1+(rand (rsd)*rand (rsd)*rand (rsd)*rand (rsd))), 0.03
                                Point_At_Trans (n_a [c])
                                translate h_a [c]
                        }
                        cylinder {
                                h_a [c], p_a [c], 0.14
                        }
                }
                #local c = c+1;
        #end
}

object {
        set_up5_3 
        scale 0.065 
        rotate y*-150
        translate <2.1,0,0>
        material {T_2317}              
}
